#!/bin/bash
# 
# EeePC CPU Front Side Bus control utility
# Andrew Wyatt
# Generic FSB control utility to shift between modes
#

. /etc/default/eeepc-acpi

function eeepc_notify {
/usr/bin/notify-send -i gnome-cpu-frequency-applet -t 1500 "EeePC $EEEPC_MODEL" "$1"
}

function cleanup {
  if [ -e "/tmp/eeepc-cpu-control-busy" ]; then
    rm -f /tmp/eeepc-cpu-control-busy 2>/dev/null
  fi
}

if [ -e "/tmp/eeepc-cpu-control-busy" ]; then
  exit 0;
else
  touch "/tmp/eeepc-cpu-control-busy"
fi

if [[ "$EEEPC_CPU" =~ "Celeron" ]]; then
  if [[ "$EEEPC_MODEL" =~ "7" ]]; then
    # Celerons in the 700 series
    FSB_NATIVE=100
    FSB_MULTIPLIER=24
    FSB_TYPE="CEL7"
  else
    # Celerons in the 900 or 1000 series
    # i2c registers at address 0x69 show 0f 1e; which translates too 30/15. Super
    # High Performance mode in Windows indicates FSB speed is 450MHz proving this accurate.
    FSB_NATIVE=30
    FSB_MULTIPLIER=15
    FSB_TYPE="CEL9"
  fi
else
  # Atom in the 900 or 1000 series
  FSB_NATIVE=110
  FSB_MULTIPLIER=25
  FSB_TYPE="ATOM"
fi

if [ ! "$OVERRIDE_FSB" = "" ]; then
  FSB_NATIVE=$OVERRIDE_FSB
fi

if [ ! "$OVERRIDE_MULTIPLIER" = "" ]; then
  FSB_MULTIPLIER=$OVERRIDE_MULTIPLIER
fi

if [ ! -e "/sys/devices/system/cpu/cpu0/cpufreq/scaling_governor" ]; then
  eeepc_notify "CPU Scaling not configured properly, can not change FSB."
  cleanup
  exit 1
fi

FSB_CPU0=$(cat /sys/devices/system/cpu/cpu0/cpufreq/scaling_governor)
if [ -e "/sys/devices/system/cpu/cpu1/cpufreq/scaling_governor" ]; then
  FSB_CPU1=$(cat /sys/devices/system/cpu/cpu1/cpufreq/scaling_governor)
fi

if [ ! -e "/proc/eee/fsb" ]; then
  eeepc_notify "EeePC FSB module not loaded, can not change FSB."
  cleanup
  exit 1
fi

FSB_CONTROL="/proc/eee/fsb"
FSB_STATE=$(cat $FSB_CONTROL)
FSB_CURRENT=$(echo $FSB_STATE | awk '{printf $1}')

if [ ! -d "$EEEPC_VAR" ]; then
  mkdir $EEEPC_VAR 2>/dev/null
fi

if [ -e "$EEEPC_VAR/cpu_mode" ]; then
  FSB_SAVED=$(cat $EEEPC_VAR/cpu_mode)
else
  FSB_SAVED=$FSB_CPU0
fi

function fsb_mode {

  if [ ! "$DONOT_SCALE_FSB" = "1" ]; then
    if [[ "$FSB_TYPE" =~ "CEL9" ]]; then
      FSB_SCALED=$(echo "$FSB_NATIVE-($FSB_NATIVE*1/6.5)" | bc)
    elif [[ "$FSB_TYPE" =~ "CEL7" ]]; then
      FSB_SCALED=$(echo "$FSB_NATIVE-($FSB_NATIVE*1/6.5)" | bc)
    elif [[ "$FSB_TYPE" =~ "ATOM" ]]; then
      FSB_SCALED=$(echo "$FSB_NATIVE-($FSB_NATIVE*1/3.1)" | bc)
    fi
  fi

  echo $1 > /sys/devices/system/cpu/cpu0/cpufreq/scaling_governor
  if [ -e "/sys/devices/system/cpu/cpu1/cpufreq/scaling_governor" ]; then
    echo $1 > /sys/devices/system/cpu/cpu1/cpufreq/scaling_governor
  fi
  eeepc_notify "CPU Mode: \"$1\""
  if [ "$1" = "performance" ]; then
    if [ "$FSB_CURRENT" < $FSB_NATIVE ]; then
      for i in `seq $FSB_CURRENT 1 $FSB_NATIVE`; do
        echo "$i $FSB_MULTIPLIER 1" >$FSB_CONTROL
        sleep 0.05
      done
    fi
  elif [ "$1" = "ondemand" ]; then
    if [ "$FSB_CURRENT" < $FSB_NATIVE ]; then
      for i in `seq $FSB_CURRENT 1 $FSB_NATIVE`; do
        echo "$i $FSB_MULTIPLIER 0" >$FSB_CONTROL
        sleep 0.05
      done
    else
      echo "$FSB_NATIVE $FSB_MULTIPLIER 0" >$FSB_CONTROL
    fi
  elif [ "$1" = "powersave" ]; then
    if [ "$FSB_CURRENT" > "$FSB_NATIVE" ]; then
      for i in `seq $FSB_NATIVE -1 $FSB_SCALED`; do
        echo "$i $FSB_MULTIPLIER 0" >$FSB_CONTROL
        sleep 0.05
      done
    fi
  else
    echo "$FSB_SCALED $FSB_MULTIPLIER 0" >$FSB_CONTROL
  fi

  echo $1 >$EEEPC_VAR/cpu_mode
}

function fsb_toggle {
  if [ $FSB_SAVED = "performance" ]; then
    fsb_mode "ondemand"
  elif [ $FSB_SAVED = "ondemand" ]; then
    fsb_mode "powersave"
  elif [ $FSB_SAVED = "powersave" ]; then
    fsb_mode "performance"
  fi
}

case $1 in
  restore)
    fsb_mode $FSB_SAVED
  ;;
  performance)
    fsb_mode "performance"
  ;;
  ondemand)
    fsb_mode "ondemand"
  ;;
  powersave)
    fsb_mode "powersave"
  ;;
  *)
    fsb_toggle
  ;;
esac
cleanup
